const ANIMATE_TIME = 500;
function swipe(element, direction, isShadow) {
    let start = 'translateX(0)';
    let end = 'translateX(0)';
    switch (direction) {
        case 'left-to-main':
            start = 'translateX(-100%)';
            break;
        case 'right-to-main':
            start = 'translateX(100%)';
            break;
        case 'main-to-left':
            end = 'translateX(-100%)';
            break;
        case 'main-to-right':
            end = 'translateX(100%)';
            break;
    }
    if (isShadow) {
        start += ' translateY(-100%)';
        end += ' translateY(-100%)';
    }
    disableScroll();
    element.style.transform = start;
    element.style.opacity = direction.startsWith('main-to') ? '1' : '0';
    requestAnimationFrame(() => {
        requestAnimationFrame(() => {
            element.style.transition = `all ${ANIMATE_TIME}ms`;
            element.style.transform = end;
            element.style.opacity = direction.startsWith('main-to') ? '0' : '1';
            enableScroll();
        });
    });
}
function disableScroll() {
    document.body.style.overflow = 'hidden';
}
function enableScroll() {
    document.body.style.overflow = '';
}
class CalendarAnimation {
    constructor() {
        this.prevState = {
            el: document.createElement('div')
        };
        this.endDirection = '';
    }
    animate(parentEle, direction) {
        if (!parentEle)
            return;
        let observer = null;
        const tableWrapper = parentEle.querySelector('.fc-view'); // Move tableWrapper outside to avoid duplicate querySelector calls
        const observeTable = () => {
            const table = parentEle.querySelector('.fc-view table');
            if (table && observer) { // Check if observer is not null before disconnecting
                observer.disconnect();
                this.performAnimation(table, tableWrapper, this.endDirection.length ? this.endDirection : direction);
            }
        };
        const listEmpty = parentEle.querySelector('.fc-list-empty');
        const table = parentEle.querySelector('.fc-view table');
        if (listEmpty || !table) { // Combine conditions to avoid duplicate code
            this.performAnimation(listEmpty || table, tableWrapper, direction);
            observer = new MutationObserver(observeTable);
            observer.observe(parentEle, { childList: true, subtree: true });
        }
        else {
            this.performAnimation(table, tableWrapper, direction);
            this.endDirection = direction;
        }
    }
    performAnimation(table, tableWrapper, direction) {
        const shadowTable = this.prevState.el;
        shadowTable.classList.add('shadow-table');
        tableWrapper.appendChild(shadowTable);
        const clonedTable = table.cloneNode(true);
        this.prevState.el = clonedTable;
        table.style.backgroundColor = 'white';
        table.style.transition = '';
        shadowTable.style.backgroundColor = 'white';
        tableWrapper.style.overflow = 'hidden';
        const swipeDirection = direction === 'next' ? 'right-to-main' : 'left-to-main';
        const shadowSwipeDirection = direction === 'next' ? 'main-to-left' : 'main-to-right';
        swipe(table, swipeDirection, false);
        swipe(shadowTable, shadowSwipeDirection, true);
        setTimeout(() => {
            table.style.backgroundColor = '';
            table.style.transform = '';
            tableWrapper.style.overflow = '';
        }, ANIMATE_TIME);
        shadowTable.remove();
    }
}
export default CalendarAnimation;
//# sourceMappingURL=calendar-animation.js.map