const ANIMATE_TIME = 500;

function swipe(element: HTMLElement, direction: 'left-to-main' | 'right-to-main' | 'main-to-left' | 'main-to-right', isShadow: boolean) {
    let start = 'translateX(0)';
    let end = 'translateX(0)';
    switch (direction) {
        case 'left-to-main':
            start = 'translateX(-100%)';
            break;
        case 'right-to-main':
            start = 'translateX(100%)';
            break;
        case 'main-to-left':
            end = 'translateX(-100%)';
            break;
        case 'main-to-right':
            end = 'translateX(100%)';
            break;
    }
    if (isShadow) {
        start += ' translateY(-100%)';
        end += ' translateY(-100%)';
    }
    disableScroll();
    element.style.transform = start;
    element.style.opacity = direction.startsWith('main-to') ? '1' : '0';
    requestAnimationFrame(() => {
        requestAnimationFrame(() => {
            element.style.transition = `all ${ANIMATE_TIME}ms`;
            element.style.transform = end;
            element.style.opacity = direction.startsWith('main-to') ? '0' : '1';
            enableScroll();
        });
    });
}

function disableScroll() {
    document.body.style.overflow = 'hidden';
}

function enableScroll() {
    document.body.style.overflow = '';
}


class CalendarAnimation {
    prevState: { el: HTMLElement; };
    endDirection : string
    constructor() {
        this.prevState = {
            el: document.createElement('div') as HTMLElement
        }
        this.endDirection = ''
    }

    animate(parentEle: HTMLElement, direction: string) {
        if (!parentEle) return;
        let observer: MutationObserver | null = null;
        const tableWrapper = parentEle.querySelector('.fc-view') as HTMLElement; // Move tableWrapper outside to avoid duplicate querySelector calls
    
        const observeTable = () => {
            const table = parentEle.querySelector('.fc-view table') as HTMLElement;
            if (table && observer) { // Check if observer is not null before disconnecting
                observer.disconnect();
                this.performAnimation(table, tableWrapper, this.endDirection.length ? this.endDirection : direction);
            }
        };
    
        const listEmpty = parentEle.querySelector('.fc-list-empty') as HTMLElement;
        const table = parentEle.querySelector('.fc-view table') as HTMLElement;
    
        if (listEmpty || !table) { // Combine conditions to avoid duplicate code
            this.performAnimation(listEmpty || table, tableWrapper, direction);
            observer = new MutationObserver(observeTable);
            observer.observe(parentEle, { childList: true, subtree: true });
        } else {
            this.performAnimation(table, tableWrapper, direction);
            this.endDirection = direction
        }
    }
    
    performAnimation(table: HTMLElement, tableWrapper: HTMLElement, direction: string ) {
        const shadowTable = this.prevState.el;
        shadowTable.classList.add('shadow-table');
    
        tableWrapper.appendChild(shadowTable);
        const clonedTable = table.cloneNode(true) as HTMLElement;
    
        this.prevState.el = clonedTable;
    
        table.style.backgroundColor = 'white';
        table.style.transition = '';
    
        shadowTable.style.backgroundColor = 'white';
        tableWrapper.style.overflow = 'hidden';
    
        const swipeDirection = direction === 'next' ? 'right-to-main' : 'left-to-main';
        const shadowSwipeDirection = direction === 'next' ? 'main-to-left' : 'main-to-right';
    
        swipe(table, swipeDirection, false);
        swipe(shadowTable, shadowSwipeDirection, true);
    
        setTimeout(() => {
            table.style.backgroundColor = '';
            table.style.transform = '';
            tableWrapper.style.overflow = '';    
        }, ANIMATE_TIME);
    
        shadowTable.remove();
    }
}

export default CalendarAnimation;
